#!BPY
# $Id$

"""
Name: 'MSP GL Mesh (.mesh)...'
Blender: 244
Group: 'Export'
"""

import sys
import math
import bpy
import Blender

class Edge:
	def __init__(self, me):
		self._medge=me
		self.faces=[]
		self.smooth=False

	def __getattr__(self, attr):
		return getattr(self._medge, attr)

	def check_smooth(self, limit):
		if len(self.faces)!=2:
			return

		d=Blender.Mathutils.DotVecs(self.faces[0].no, self.faces[1].no)
		if (d>limit and self.faces[0].smooth and self.faces[1].smooth) or d>0.999:
			self.smooth=1

	def other_face(self, f):
		if f.index==self.faces[0].index:
			if len(self.faces)>=2:
				return self.faces[1]
			else:
				return None
		else:
			return self.faces[0]


class Vertex:
	def __init__(self, mv):
		if mv.__class__==Vertex:
			self._mvert=mv._mvert
		else:
			self._mvert=mv

	def __getattr__(self, attr):
		return getattr(self._mvert, attr)


class Face:
	def __init__(self, mf):
		self._mface=mf
		self.smooth_group=None
		self.edges=[]
		self.verts=[v for v in mf.verts]
		self.flag=False

	def __getattr__(self, attr):
		return getattr(self._mface, attr)

	def get_following_vertex(self, *vt):
		seen=False
		indices=[v.index for v in vt]
		for v in self.verts:
			if v.index in indices:
				seen=True
			elif seen:
				return v

		if seen:
			return self.verts[0]

		return None


class SmoothGroup:
	def __init__(self, index):
		self.index=index
		self.faces=[]
		self.verts=[]

	def find_vertices(self):
		vert_map={}
		for f in self.faces:
			for i in range(len(f.verts)):
				v=f.verts[i]
				if v.index not in vert_map:
					vt=Vertex(v)
					if not f.smooth:
						vt.no=f.no
					self.verts.append(vt)
					vert_map[v.index]=vt
					f.verts[i]=vt
				else:
					f.verts[i]=vert_map[v.index]


class Exporter:
	def __init__(self, fn):
		self.filename=fn
		if fn==None:
			self.out_file=sys.stdout
		else:
			self.out_file=file(fn, "w")
		self.use_strips=True
		self.use_degen_tris=True

	def find_smooth_group(self, face, sg):
		face.smooth_group=sg
		sg.faces.append(face)
		queue=[face]
		while queue:
			cur=queue.pop(0)
			for e in cur.edges:
				if e.smooth:
					other=e.other_face(cur)
					if other and not other.smooth_group:
						other.smooth_group=sg
						sg.faces.append(other)
						queue.append(other)

	def create_strip(self, face):
		edge=None
		for e in face.edges:
			other=e.other_face(face)
			if other and other.smooth_group.index==face.smooth_group.index and not other.flag:
				edge=e
				break

		if not edge:
			return None

		v1=face.get_following_vertex(edge.v1, edge.v2)
		v2=face.get_following_vertex(v1)
		if len(face.verts)==4:
			result=[v2, v1]
		else:
			result=[v1, v2]

		while 1:
			face.flag=True
			for i in range(2, len(face.verts)):
				v=face.get_following_vertex(result[-2], result[-1])
				result.append(v)

			i1=result[-2].index
			i2=result[-1].index
			ekey=(min(i1, i2), max(i1, i2))
			for e in face.edges:
				if e.key==ekey:
					edge=e
					break

			next=edge.other_face(face)
			if next.smooth_group.index!=face.smooth_group.index or next.flag:
				break
			face=next

		return result

	def export(self):
		scene=bpy.data.scenes.active

		obj=scene.objects.active
		if obj.getType()!="Mesh":
			raise Exception, "Can only export Mesh data"

		mesh=obj.getData(mesh=True)

		faces=[Face(f) for f in mesh.faces]

		edges=dict([(e.key, Edge(e)) for e in mesh.edges])
		for f in faces:
			for e in f.edge_keys:
				edges[e].faces.append(f)
				f.edges.append(edges[e])

		smooth_limit=math.cos(mesh.degr*math.pi/180)
		for e in edges.itervalues():
			e.check_smooth(smooth_limit)

		smooth_groups=[]
		for f in faces:
			if not f.smooth_group:
				sg=SmoothGroup(len(smooth_groups))
				smooth_groups.append(sg)
				self.find_smooth_group(f, sg)

		for sg in smooth_groups:
			sg.find_vertices()

		strips=[]
		if self.use_strips:
			for sg in smooth_groups:
				for f in sg.faces:
					if not f.flag:
						strip=self.create_strip(f)
						if strip:
							strips.append(strip)

			if self.use_degen_tris:
				big_strip=[]
				for s in strips:
					if big_strip:
						big_strip+=[big_strip[-1], s[0]]
					big_strip+=s

				for f in faces:
					if not f.flag:
						if big_strip:
							big_strip+=[big_strip[-1], f.verts[0]]
						big_strip+=[f.verts[i] for i in (0, 1, -1)]
						if len(f.verts)==4:
							big_strip.append(f.verts[-2])
						f.flag=True

				strips=[big_strip]

		verts=[]
		for sg in smooth_groups:
			for v in sg.verts:
				v.index=len(verts)
				verts.append(v)

		self.out_file.write("vertices NORMAL3_VERTEX3\n{\n")
		for v in verts:
			self.out_file.write("\tnormal3 %f %f %f;\n"%tuple(v.no))
			self.out_file.write("\tvertex3 %f %f %f;\n"%tuple(v.co))
		self.out_file.write("};\n")
		for s in strips:
			self.out_file.write("batch TRIANGLE_STRIP\n{\n\tindices")
			n=0
			for v in s:
				self.out_file.write(" %u"%v.index)
				n+=1;
				if n%32==0:
					self.out_file.write(";\n\tindices")
			self.out_file.write(";\n};\n")

		first=True
		for f in faces:
			if not f.flag:
				if first:
					self.out_file.write("batch TRIANGLES\n{\n")
					first=False
				for i in range(2, len(f.verts)):
					self.out_file.write("\tindices %u %u %u;\n"%(f.verts[0].index, f.verts[i-1].index, f.verts[i].index))
		if not first:
			self.out_file.write("};\n")


class FrontEnd:
	def run(self):
		#self.export(None)
		Blender.Window.FileSelector(self.export, "Export MSP GL mesh", Blender.sys.makename(ext='.mesh'))

	def export(self, fn):
		exp=Exporter(fn)
		exp.export()


if __name__=="__main__":
	fe=FrontEnd()
	fe.run()
